<?php
/**
 * Estonian PHPMailer language file: refer to English translation for definitive list
 * @package PHPMailer
 * @author Indrek Päri
 * @author Elan Ruusamäe <glen@delfi.ee>
 */

$PHPMAILER_LANG['authenticate']         = 'SMTP Viga: Autoriseerimise viga.';
$PHPMAILER_LANG['connect_host']         = 'SMTP Viga: Ei õnnestunud luua ühendust SMTP serveriga.';
$PHPMAILER_LANG['data_not_accepted']    = 'SMTP Viga: Vigased andmed.';
$PHPMAILER_LANG['empty_message']        = 'Tühi kirja sisu';
$PHPMAILER_LANG["encoding"]             = 'Tundmatu kodeering: ';
$PHPMAILER_LANG['execute']              = 'Tegevus ebaõnnestus: ';
$PHPMAILER_LANG['file_access']          = 'Pole piisavalt õiguseid järgneva faili avamiseks: ';
$PHPMAILER_LANG['file_open']            = 'Faili Viga: Faili avamine ebaõnnestus: ';
$PHPMAILER_LANG['from_failed']          = 'Järgnev saatja e-posti aadress on vigane: ';
$PHPMAILER_LANG['instantiate']          = 'mail funktiooni käivitamine ebaõnnestus.';
$PHPMAILER_LANG['invalid_address']      = 'Saatmine peatatud, e-posti address vigane: ';
$PHPMAILER_LANG['provide_address']      = 'Te peate määrama vähemalt ühe saaja e-posti aadressi.';
$PHPMAILER_LANG['mailer_not_supported'] = ' maileri tugi puudub.';
$PHPMAILER_LANG['recipients_failed']    = 'SMTP Viga: Järgnevate saajate e-posti aadressid on vigased: ';
$PHPMAILER_LANG["signing"]              = 'Viga allkirjastamisel: ';
$PHPMAILER_LANG['smtp_connect_failed']  = 'SMTP Connect() ebaõnnestus.';
$PHPMAILER_LANG['smtp_error']           = 'SMTP serveri viga: ';
$PHPMAILER_LANG['variable_set']         = 'Ei õnnestunud määrata või lähtestada muutujat: ';
$PHPMAILER_LANG['extension_missing']    = 'Nõutud laiendus on puudu: ';
