<?php
/**
 * French PHPMailer language file: refer to English translation for definitive list
 * @package PHPMailer
 * Some French punctuation requires a thin non-breaking space (U+202F) character before it,
 * for example before a colon or exclamation mark.
 * There is one of these characters between these quotes: " "
 * @link http://unicode.org/udhr/n/notes_fra.html
 */

$PHPMAILER_LANG['authenticate']         = 'Erreur SMTP : échec de l\'authentification.';
$PHPMAILER_LANG['connect_host']         = 'Erreur SMTP : impossible de se connecter au serveur SMTP.';
$PHPMAILER_LANG['data_not_accepted']    = 'Erreur SMTP : données incorrectes.';
$PHPMAILER_LANG['empty_message']        = 'Corps du message vide.';
$PHPMAILER_LANG['encoding']             = 'Encodage inconnu : ';
$PHPMAILER_LANG['execute']              = 'Impossible de lancer l\'exécution : ';
$PHPMAILER_LANG['file_access']          = 'Impossible d\'accéder au fichier : ';
$PHPMAILER_LANG['file_open']            = 'Ouverture du fichier impossible : ';
$PHPMAILER_LANG['from_failed']          = 'L\'adresse d\'expéditeur suivante a échoué : ';
$PHPMAILER_LANG['instantiate']          = 'Impossible d\'instancier la fonction mail.';
$PHPMAILER_LANG['invalid_address']      = 'L\'adresse courriel n\'est pas valide : ';
$PHPMAILER_LANG['mailer_not_supported'] = ' client de messagerie non supporté.';
$PHPMAILER_LANG['provide_address']      = 'Vous devez fournir au moins une adresse de destinataire.';
$PHPMAILER_LANG['recipients_failed']    = 'Erreur SMTP : les destinataires suivants sont en erreur : ';
$PHPMAILER_LANG['signing']              = 'Erreur de signature : ';
$PHPMAILER_LANG['smtp_connect_failed']  = 'Échec de la connexion SMTP.';
$PHPMAILER_LANG['smtp_error']           = 'Erreur du serveur SMTP : ';
$PHPMAILER_LANG['variable_set']         = 'Impossible d\'initialiser ou de réinitialiser une variable : ';
$PHPMAILER_LANG['extension_missing']    = 'Extension manquante : ';
