<?php
/**
 * Ukrainian PHPMailer language file: refer to English translation for definitive list
 * @package PHPMailer
 * @author Yuriy Rudyy <yrudyy@prs.net.ua>
 * @fixed by Boris Yurchenko <boris@yurchenko.pp.ua>
 */

$PHPMAILER_LANG['authenticate']         = 'Помилка SMTP: помилка авторизації.';
$PHPMAILER_LANG['connect_host']         = 'Помилка SMTP: не вдається під\'єднатися до серверу SMTP.';
$PHPMAILER_LANG['data_not_accepted']    = 'Помилка SMTP: дані не прийняті.';
$PHPMAILER_LANG['encoding']             = 'Невідомий тип кодування: ';
$PHPMAILER_LANG['execute']              = 'Неможливо виконати команду: ';
$PHPMAILER_LANG['file_access']          = 'Немає доступу до файлу: ';
$PHPMAILER_LANG['file_open']            = 'Помилка файлової системи: не вдається відкрити файл: ';
$PHPMAILER_LANG['from_failed']          = 'Невірна адреса відправника: ';
$PHPMAILER_LANG['instantiate']          = 'Неможливо запустити функцію mail.';
$PHPMAILER_LANG['provide_address']      = 'Будь-ласка, введіть хоча б одну адресу e-mail отримувача.';
$PHPMAILER_LANG['mailer_not_supported'] = ' - поштовий сервер не підтримується.';
$PHPMAILER_LANG['recipients_failed']    = 'Помилка SMTP: відправлення наступним отримувачам не вдалося: ';
$PHPMAILER_LANG['empty_message']        = 'Пусте тіло повідомлення';
$PHPMAILER_LANG['invalid_address']      = 'Не відправлено, невірний формат адреси e-mail: ';
$PHPMAILER_LANG['signing']              = 'Помилка підпису: ';
$PHPMAILER_LANG['smtp_connect_failed']  = 'Помилка з\'єднання із SMTP-сервером';
$PHPMAILER_LANG['smtp_error']           = 'Помилка SMTP-сервера: ';
$PHPMAILER_LANG['variable_set']         = 'Неможливо встановити або перевстановити змінну: ';
//$PHPMAILER_LANG['extension_missing']    = 'Extension missing: ';
